@@ -2,7 +2,7 @@
 // Syd: rock-solid application kernel
 // build.rs: Helper file for build-time information
 //
-// Copyright (c) 2021, 2024, 2025 Ali Polatel <alip@chesswob.org>
+// Copyright (c) 2021, 2024, 2025, 2026 Ali Polatel <alip@chesswob.org>
 //
 // SPDX-License-Identifier: GPL-3.0
 
@@ -130,5 +130,18 @@ fn main() -> Result<(), Box<dyn std::error::Error>> {
     println!("cargo:rustc-env=SYD_GIT_COMMIT={comm}");
     println!("cargo:rustc-env=SYD_GIT_HEAD={head}");
 
+    // KCOV support
+    if env::var("CARGO_FEATURE_KCOV").is_ok() {
+        println!("cargo:warning=KCOV feature enabled, compiling C stub");
+
+        // Compile the C stub without instrumentation.
+        cc::Build::new()
+            .file("src/kcov/stub.c")
+            .opt_level(2)
+            .warnings(true)
+            .compile("kcov_stub");
+
+        println!("cargo:rerun-if-changed=src/kcov/stub.c");
+    }
     Ok(())
 }
