@@ -2,7 +2,7 @@
 //
 // lib/src/syd.go: Go bindings of libsyd, the syd API C Library
 //
-// Copyright (c) 2023, 2024, 2025 Ali Polatel <alip@chesswob.org>
+// Copyright (c) 2023, 2024, 2025, 2026 Ali Polatel <alip@chesswob.org>
 //
 // SPDX-License-Identifier: LGPL-3.0
 //
@@ -37,6 +37,14 @@ const (
 	// initial process (syd exec child).
 	LockExec
 
+	// LockDrop indicates that the sandbox lock is in drop-only mode, allowing
+	// only privilege-dropping sandbox commands.
+	LockDrop
+
+	// LockRead indicates that the sandbox lock is in read-only mode, allowing
+	// only read-only access to sandbox state.
+	LockRead
+
 	// LockOn indicates that the sandbox lock is on, disallowing all sandbox commands.
 	// In this state, the sandbox is in its most restrictive mode, not permitting
 	// any operations that could modify its state or configuration.
@@ -74,6 +82,7 @@ type Sandbox struct {
 	State           string   `json:"state"`
 	Lock            string   `json:"lock"`
 	Cpid            int      `json:"cpid"`
+	DefaultFs       string   `json:"default_fs"`
 	DefaultWalk     string   `json:"default_walk"`
 	DefaultStat     string   `json:"default_stat"`
 	DefaultRead     string   `json:"default_read"`
@@ -94,6 +103,7 @@ type Sandbox struct {
 	DefaultChmod    string   `json:"default_chmod"`
 	DefaultChattr   string   `json:"default_chattr"`
 	DefaultChroot   string   `json:"default_chroot"`
+	DefaultNotify   string   `json:"default_notify"`
 	DefaultUtime    string   `json:"default_utime"`
 	DefaultMkbdev   string   `json:"default_mkbdev"`
 	DefaultMkcdev   string   `json:"default_mkcdev"`
@@ -277,6 +287,38 @@ func Exec(file string, argv []string) error {
 	return nil
 }
 
+// EnableFs enables fs sandboxing.
+//
+// Returns nil on success, and an error corresponding to the negated errno
+// on failure. The error is of type syscall.Errno.
+func EnableFs() error {
+	result := C.syd_enable_fs()
+	if result != 0 {
+		return syscall.Errno(-result)
+	}
+	return nil
+}
+
+// DisableFs disables fs sandboxing.
+//
+// Returns nil on success, and an error corresponding to the negated errno
+// on failure. The error is of type syscall.Errno.
+func DisableFs() error {
+	result := C.syd_disable_fs()
+	if result != 0 {
+		return syscall.Errno(-result)
+	}
+	return nil
+}
+
+// EnabledFs checks if fs sandboxing is enabled in the syd environment.
+//
+// It returns true if fs sandboxing is enabled, and false otherwise.
+func EnabledFs() bool {
+	result := C.syd_enabled_fs()
+	return bool(result)
+}
+
 // EnableWalk enables walk sandboxing.
 //
 // Returns nil on success, and an error corresponding to the negated errno
@@ -919,6 +961,38 @@ func EnabledChroot() bool {
 	return bool(result)
 }
 
+// EnableNotify enables notify sandboxing.
+//
+// Returns nil on success, and an error corresponding to the negated errno
+// on failure. The error is of type syscall.Errno.
+func EnableNotify() error {
+	result := C.syd_enable_notify()
+	if result != 0 {
+		return syscall.Errno(-result)
+	}
+	return nil
+}
+
+// DisableNotify disables notify sandboxing.
+//
+// Returns nil on success, and an error corresponding to the negated errno
+// on failure. The error is of type syscall.Errno.
+func DisableNotify() error {
+	result := C.syd_disable_notify()
+	if result != 0 {
+		return syscall.Errno(-result)
+	}
+	return nil
+}
+
+// EnabledNotify checks if notify sandboxing is enabled in the syd environment.
+//
+// It returns true if notify sandboxing is enabled, and false otherwise.
+func EnabledNotify() bool {
+	result := C.syd_enabled_notify()
+	return bool(result)
+}
+
 // EnableUtime enables utime sandboxing.
 //
 // Returns nil on success, and an error corresponding to the negated errno
@@ -1264,6 +1338,18 @@ func EnabledTPE() bool {
 	return bool(result)
 }
 
+// Set default action for Filesystem sandboxing.
+//
+// Returns nil on success, and an error corresponding to the negated errno
+// on failure. The error is of type syscall.Errno.
+func DefaultFs(action Action) error {
+	result := C.syd_default_fs(C.action_t(action))
+	if result != 0 {
+		return syscall.Errno(-result)
+	}
+	return nil
+}
+
 // Set default action for Walk sandboxing.
 //
 // Returns nil on success, and an error corresponding to the negated errno
@@ -1504,6 +1590,18 @@ func DefaultChroot(action Action) error {
 	return nil
 }
 
+// Set default action for Notify sandboxing.
+//
+// Returns nil on success, and an error corresponding to the negated errno
+// on failure. The error is of type syscall.Errno.
+func DefaultNotify(action Action) error {
+	result := C.syd_default_notify(C.action_t(action))
+	if result != 0 {
+		return syscall.Errno(-result)
+	}
+	return nil
+}
+
 // Set default action for Utime sandboxing.
 //
 // Returns nil on success, and an error corresponding to the negated errno
@@ -1660,6 +1758,54 @@ func IoctlDeny(request uint64) error {
 	return nil
 }
 
+// FsAdd adds the specified glob pattern to the given actionlist of
+// Filesystem sandboxing.
+//
+// Returns nil on success, and an error corresponding to the negated errno
+// on failure. The error is of type syscall.Errno.
+func FsAdd(action Action, glob string) error {
+	cGlob := C.CString(glob)
+	defer C.free(unsafe.Pointer(cGlob))
+
+	result := C.syd_fs_add(C.action_t(action), cGlob)
+	if result != 0 {
+		return syscall.Errno(-result)
+	}
+	return nil
+}
+
+// FsDel removes the first instance from the end of the given
+// actionlist of read sandboxing.
+//
+// Returns nil on success, and an error corresponding to the negated errno
+// on failure. The error is of type syscall.Errno.
+func FsDel(action Action, glob string) error {
+	cGlob := C.CString(glob)
+	defer C.free(unsafe.Pointer(cGlob))
+
+	result := C.syd_fs_del(C.action_t(action), cGlob)
+	if result != 0 {
+		return syscall.Errno(-result)
+	}
+	return nil
+}
+
+// FsRem removes all matching patterns from the given actionlist of
+// Filesystem sandboxing.
+//
+// Returns nil on success, and an error corresponding to the negated errno
+// on failure. The error is of type syscall.Errno.
+func FsRem(action Action, glob string) error {
+	cGlob := C.CString(glob)
+	defer C.free(unsafe.Pointer(cGlob))
+
+	result := C.syd_fs_rem(C.action_t(action), cGlob)
+	if result != 0 {
+		return syscall.Errno(-result)
+	}
+	return nil
+}
+
 // WalkAdd adds the specified glob pattern to the given actionlist of
 // Walk sandboxing.
 //
@@ -1900,54 +2046,6 @@ func ExecRem(action Action, glob string) error {
 	return nil
 }
 
-// IoctlAdd adds the specified glob pattern to the given actionlist of
-// Ioctl sandboxing.
-//
-// Returns nil on success, and an error corresponding to the negated errno
-// on failure. The error is of type syscall.Errno.
-func IoctlAdd(action Action, glob string) error {
-	cGlob := C.CString(glob)
-	defer C.free(unsafe.Pointer(cGlob))
-
-	result := C.syd_ioctl_add(C.action_t(action), cGlob)
-	if result != 0 {
-		return syscall.Errno(-result)
-	}
-	return nil
-}
-
-// IoctlDel removes the first instance from the end of the given
-// actionlist of read sandboxing.
-//
-// Returns nil on success, and an error corresponding to the negated errno
-// on failure. The error is of type syscall.Errno.
-func IoctlDel(action Action, glob string) error {
-	cGlob := C.CString(glob)
-	defer C.free(unsafe.Pointer(cGlob))
-
-	result := C.syd_ioctl_del(C.action_t(action), cGlob)
-	if result != 0 {
-		return syscall.Errno(-result)
-	}
-	return nil
-}
-
-// IoctlRem removes all matching patterns from the given actionlist of
-// Ioctl sandboxing.
-//
-// Returns nil on success, and an error corresponding to the negated errno
-// on failure. The error is of type syscall.Errno.
-func IoctlRem(action Action, glob string) error {
-	cGlob := C.CString(glob)
-	defer C.free(unsafe.Pointer(cGlob))
-
-	result := C.syd_ioctl_rem(C.action_t(action), cGlob)
-	if result != 0 {
-		return syscall.Errno(-result)
-	}
-	return nil
-}
-
 // CreateAdd adds the specified glob pattern to the given actionlist of
 // Create sandboxing.
 //
@@ -2620,6 +2718,54 @@ func ChrootRem(action Action, glob string) error {
 	return nil
 }
 
+// NotifyAdd adds the specified glob pattern to the given actionlist of
+// Notify sandboxing.
+//
+// Returns nil on success, and an error corresponding to the negated errno
+// on failure. The error is of type syscall.Errno.
+func NotifyAdd(action Action, glob string) error {
+	cGlob := C.CString(glob)
+	defer C.free(unsafe.Pointer(cGlob))
+
+	result := C.syd_notify_add(C.action_t(action), cGlob)
+	if result != 0 {
+		return syscall.Errno(-result)
+	}
+	return nil
+}
+
+// NotifyDel removes the first instance from the end of the given
+// actionlist of notify sandboxing.
+//
+// Returns nil on success, and an error corresponding to the negated errno
+// on failure. The error is of type syscall.Errno.
+func NotifyDel(action Action, glob string) error {
+	cGlob := C.CString(glob)
+	defer C.free(unsafe.Pointer(cGlob))
+
+	result := C.syd_notify_del(C.action_t(action), cGlob)
+	if result != 0 {
+		return syscall.Errno(-result)
+	}
+	return nil
+}
+
+// NotifyRem removes all matching patterns from the given actionlist of
+// Notify sandboxing.
+//
+// Returns nil on success, and an error corresponding to the negated errno
+// on failure. The error is of type syscall.Errno.
+func NotifyRem(action Action, glob string) error {
+	cGlob := C.CString(glob)
+	defer C.free(unsafe.Pointer(cGlob))
+
+	result := C.syd_notify_rem(C.action_t(action), cGlob)
+	if result != 0 {
+		return syscall.Errno(-result)
+	}
+	return nil
+}
+
 // UtimeAdd adds the specified glob pattern to the given actionlist of
 // Utime sandboxing.
 //
