@@ -72,7 +72,7 @@ const PKG_DESCRIPTION: &str = env!("CARGO_PKG_DESCRIPTION");
 const PKG_AUTHORS: &str = env!("CARGO_PKG_AUTHORS");
 const PKG_LICENSE: &str = env!("CARGO_PKG_LICENSE");
 
-const CAPSET_LPATH: &[&str] = &["walk", "stat", "chdir"];
+const CAPSET_LPATH: &[&str] = &["walk", "stat", "chdir", "notify"];
 const CAPSET_RPATH: &[&str] = &["read", "readdir"];
 const CAPSET_WPATH: &[&str] = &["write", "truncate"];
 const CAPSET_CPATH: &[&str] = &["create", "delete", "rename"];
@@ -141,6 +141,7 @@ const CAP_ORDER: &[&str] = &[
     "chmod",
     "chattr",
     "chroot",
+    "notify",
     "utime",
     "mkbdev",
     "mkcdev",
