@@ -2,7 +2,7 @@
 // Syd: rock-solid application kernel
 // src/kernel/net/mod.rs: Network syscall handlers
 //
-// Copyright (c) 2023, 2024, 2025 Ali Polatel <alip@chesswob.org>
+// Copyright (c) 2023, 2024, 2025, 2026 Ali Polatel <alip@chesswob.org>
 //
 // SPDX-License-Identifier: GPL-3.0
 
@@ -30,7 +30,7 @@ use crate::{
     },
     config::HOOK_SCKCALLS,
     confine::{op2errno, op2name, scmp_arch_bits},
-    fs::fd_status_flags,
+    fd::fd_status_flags,
     kernel::net::{
         accept::handle_accept,
         bind::handle_bind,
@@ -44,10 +44,12 @@ use crate::{
         sendto::handle_sendto,
         socket::{handle_socket, handle_socketpair},
     },
+    log_enabled,
     lookup::{file_type, safe_canonicalize, CanonicalPath, FileType, FsFlags},
     path::{XPath, XPathBuf},
     req::UNotifyEventRequest,
     sandbox::{Action, Capability, SandboxGuard},
+    syslog::LogLevel,
     warn,
 };
 
@@ -204,10 +206,13 @@ fn syscall_network_handler(request: UNotifyEventRequest, args: &[u64; 6], op: u8
     syscall_handler!(request, |request: UNotifyEventRequest| {
         let sandbox = request.get_sandbox();
         let flags = *sandbox.flags;
-        let allow_safe_bind = flags.allow_safe_bind();
-        let allow_safe_kcapi = flags.allow_safe_kcapi();
-        let allow_unsupp_socket = flags.allow_unsupp_socket();
-        let restrict_oob = !flags.allow_unsafe_oob();
+        let options = *sandbox.options;
+        let allow_safe_bind = options.allow_safe_bind();
+        let allow_safe_kcapi = options.allow_safe_kcapi();
+        let allow_unsupp_socket = options.allow_unsupp_socket();
+        let restrict_oob = !options.allow_unsafe_oob();
+        let restrict_recvmsg = !options.allow_unsafe_recvmsg();
+        let randomize_fds = flags.force_rand_fd();
 
         let cap = match op {
             0x1 => {
@@ -215,7 +220,7 @@ fn syscall_network_handler(request: UNotifyEventRequest, args: &[u64; 6], op: u8
                 // b. socketcall -> socket indirection.
                 let nlfam = sandbox.netlink_families;
                 drop(sandbox); // drop read-lock before emulation.
-                return handle_socket(&request, args, flags, nlfam);
+                return handle_socket(&request, args, flags, options, nlfam);
             }
             0x8 => {
                 // a. socketpair system call.
@@ -224,6 +229,12 @@ fn syscall_network_handler(request: UNotifyEventRequest, args: &[u64; 6], op: u8
                 // access with the dummy `!unnamed' path.
                 return handle_socketpair(&request, sandbox, args, 0x8);
             }
+            0x11 | 0x13 | u8::MAX if !restrict_recvmsg => {
+                // SAFETY:
+                // 1. This can only happen by socketcall(2) indirection.
+                // 2. No pointer dereference in access check.
+                return Ok(unsafe { request.continue_syscall() });
+            }
             0x5 | 0x6 | 0x7 | 0xc | 0x11 | 0x12 | 0x13 | u8::MAX => {
                 // accept, accept4, getsockname, getpeername:
                 //
@@ -282,27 +293,27 @@ fn syscall_network_handler(request: UNotifyEventRequest, args: &[u64; 6], op: u8
             }
             0x11 => {
                 // recvmsg is used for informational purposes only.
-                return handle_recvmsg(fd, &request, args, flags);
+                return handle_recvmsg(fd, &request, args, flags, options);
             }
             0x13 => {
                 // recvmsg is used for informational purposes only.
-                return handle_recvmmsg(fd, &request, args, flags);
+                return handle_recvmmsg(fd, &request, args, flags, options);
             }
             u8::MAX => {
                 // recvmmsg_time64 is used for informational purposes only.
-                return handle_recvmmsg64(fd, &request, args, flags);
+                return handle_recvmmsg64(fd, &request, args, flags, options);
             }
             0xf => {
                 // getsockopt is used for informational purposes only.
-                return handle_getsockopt(fd, &request, args);
+                return handle_getsockopt(fd, &request, args, randomize_fds);
             }
             0x10 => {
                 // sendmsg uses a different data structure, so we handle it in its own branch.
-                return handle_sendmsg(fd, &request, args, flags);
+                return handle_sendmsg(fd, &request, args, flags, options);
             }
             0x14 => {
                 // sendmmsg uses a different data structure, so we handle it in its own branch.
-                return handle_sendmmsg(fd, &request, args, flags);
+                return handle_sendmmsg(fd, &request, args, flags, options);
             }
             _ => {} // fall through.
         }
@@ -346,7 +357,7 @@ fn syscall_network_handler(request: UNotifyEventRequest, args: &[u64; 6], op: u8
                 // Turn 0.0.0.0 into 127.0.0.1 unless trace/allow_unsafe_any_addr:1.
                 // Turn all addresses to 127.0.0.1 if trace/force_local_net:1.
                 if !anyaddr {
-                    make_loaddr(&mut addr, local_net);
+                    make_loaddr(&mut addr, local_net, op)?;
                 }
                 // Check for access.
                 sandbox_addr(&request, &sandbox, &addr, &root, op, cap).map_err(|errno| {
@@ -361,7 +372,7 @@ fn syscall_network_handler(request: UNotifyEventRequest, args: &[u64; 6], op: u8
                 // Turn :: into :1 unless trace/allow_unsafe_any_addr:1.
                 // Turn all addresses to ::1 if trace/force_local_net:1.
                 if !anyaddr {
-                    make_lo6addr(&mut addr, local_net);
+                    make_lo6addr(&mut addr, local_net, op)?;
                 }
                 // Check for access.
                 sandbox_addr(&request, &sandbox, &addr, &root, op, cap).map_err(|errno| {
@@ -456,12 +467,12 @@ fn get_addr(
 // Canonicalizes UNIX domain socket names.
 // Returns address and directory.
 // Directory is None for non-UNIX addresses.
-fn canon_addr<'a>(
+fn canon_addr(
     request: &UNotifyEventRequest,
     sandbox: &SandboxGuard,
     addr: &SockaddrStorage,
     cap: Capability,
-) -> Result<(SockaddrStorage, Option<CanonicalPath<'a>>), Errno> {
+) -> Result<(SockaddrStorage, Option<CanonicalPath>), Errno> {
     #[expect(clippy::cast_possible_truncation)]
     if let Some(path) = addr.as_unix_addr().and_then(|a| a.path()) {
         // Check for chroot.
@@ -491,7 +502,7 @@ fn canon_addr<'a>(
         let pid = request.scmpreq.pid();
         let path = safe_canonicalize(pid, None, &path, fsflags, Some(sandbox.deref()))?;
 
-        let path_bytes = if path.base.is_empty() {
+        let sun_path = if path.base().is_empty() {
             // SAFETY: We open a FD to the path and then use the
             // proc path /proc/thread-self/fd/$fd in address' path
             // argument to avoid symlink TOCTOU because connect and
@@ -501,7 +512,7 @@ fn canon_addr<'a>(
             let mut pfd = XPathBuf::from("/proc/thread-self/fd");
             pfd.push_fd(fd.as_raw_fd());
             pfd.append_byte(0);
-            pfd.into_vec()
+            pfd
         } else {
             // SAFETY:
             // 1. We split the address into directory and basename
@@ -513,9 +524,9 @@ fn canon_addr<'a>(
             // 2. We add "./" for easier identification at recvfrom(2)
             //    boundary which is for informational purposes only.
             let mut base = XPathBuf::from("./");
-            base.append_bytes(path.base.as_os_str().as_bytes());
+            base.append_bytes(path.base().as_os_str().as_bytes());
             base.append_byte(0);
-            base.into_vec()
+            base
         };
 
         // Create sockaddr_un struct.
@@ -523,7 +534,7 @@ fn canon_addr<'a>(
             sun_family: libc::AF_UNIX as libc::sa_family_t,
             sun_path: [0; UNIX_PATH_MAX],
         };
-        let socklen = path_bytes.len();
+        let socklen = sun_path.len();
         if socklen > UNIX_PATH_MAX {
             return Err(Errno::ENAMETOOLONG);
         }
@@ -531,7 +542,7 @@ fn canon_addr<'a>(
         // SAFETY: Copy the bytes without overlapping regions.
         unsafe {
             std::ptr::copy_nonoverlapping(
-                path_bytes.as_ptr(),
+                sun_path.as_ptr(),
                 sockaddr.sun_path.as_mut_ptr().cast(),
                 socklen,
             )
@@ -666,9 +677,9 @@ pub(crate) fn sandbox_addr_unix(
     };
 
     // Check for access.
-    let (action, filter) = sandbox.check_unix(caps, &path);
+    let action = sandbox.check_unix(caps, &path);
 
-    if !filter {
+    if action.is_logging() && log_enabled!(LogLevel::Warn) {
         let sys = op2name(op);
         if sandbox.log_scmp() {
             warn!("ctx": "access", "cap": caps, "act": action,
@@ -739,9 +750,9 @@ pub(crate) fn sandbox_addr_inet(
 
     // Check for access and IP blocklist as necessary.
     // caps.is_empty() implies accept{,4}(2) here.
-    let (action, filter) = sandbox.check_ip(caps, addr, port);
+    let action = sandbox.check_ip(caps, addr, port);
 
-    if caps.is_empty() && !filter {
+    if caps.is_empty() && action.is_logging() {
         // accept{,4}
         let ipv = if addr.is_ipv6() { 6 } else { 4 };
         let sys = op2name(op);
@@ -756,7 +767,7 @@ pub(crate) fn sandbox_addr_inet(
                 "tip": format!("configure `block-{addr}'"),
                 "pid": request.scmpreq.pid);
         }
-    } else if !filter {
+    } else if action.is_logging() {
         // connect, sendto, send{m,}msg
         let ipv = if addr.is_ipv6() { 6 } else { 4 };
         let sys = op2name(op);
@@ -788,7 +799,7 @@ pub(crate) fn sandbox_addr_inet(
 
 /// Process a socket address of an unsupported socket family.
 pub(crate) fn sandbox_addr_notsup(sandbox: &SandboxGuard) -> Result<(), Errno> {
-    if sandbox.flags.allow_unsupp_socket() {
+    if sandbox.options.allow_unsupp_socket() {
         Ok(())
     } else {
         Err(Errno::EAFNOSUPPORT)
@@ -818,9 +829,9 @@ pub(crate) fn sandbox_addr_unnamed(
         // Sandboxing is off.
         return Ok(());
     }
-    let (action, filter) = sandbox.check_unix(caps, name);
+    let action = sandbox.check_unix(caps, name);
 
-    if !filter {
+    if action.is_logging() && log_enabled!(LogLevel::Warn) {
         let sys = op2name(op);
         if sandbox.log_scmp() {
             warn!("ctx": "access", "cap": caps, "act": action,
@@ -853,24 +864,48 @@ pub(crate) fn to_msgflags(arg: u64) -> Result<MsgFlags, Errno> {
     MsgFlags::from_bits(flags).ok_or(Errno::EINVAL)
 }
 
-// Change 0.0.0.0 to 127.0.0.1 (IPv4)
+// Change 0.0.0.0 to 127.0.0.1 (IPv4).
+//
 // If `force` is true change all addresses.
-pub(crate) fn make_loaddr(addr: &mut SockaddrStorage, force: bool) {
-    if !force && !is_anyaddr(addr) {
-        return;
+pub(crate) fn make_loaddr(addr: &mut SockaddrStorage, force: bool, op: u8) -> Result<(), Errno> {
+    let is_any = is_anyaddr(addr);
+
+    // SAFETY: Deny connect/sendto to anyaddr with ENETUNREACH.
+    // This is consistent with FreeBSD's net.inet.ip.connect_inaddr_wild sysctl.
+    // 0x3: connect, 0x9: send, 0xb: sendto, 0x10: sendmsg, 0x14: sendmmsg
+    if is_any && matches!(op, 0x3 | 0x9 | 0xb | 0x10 | 0x14) {
+        return Err(Errno::ENETUNREACH);
     }
+
+    if !force && !is_any {
+        return Ok(());
+    }
+
     if let Some(sin) = addr.as_sockaddr_in() {
         let port = sin.port();
         *addr = SockaddrStorage::from(SocketAddrV4::new(Ipv4Addr::LOCALHOST, port));
     }
+
+    Ok(())
 }
 
-// Change :: to ::1 (IPv6)
+// Change :: to ::1 (IPv6).
+//
 // If `force` is true change all addresses.
-pub(crate) fn make_lo6addr(addr: &mut SockaddrStorage, force: bool) {
-    if !force && !is_any6addr(addr) {
-        return;
+pub(crate) fn make_lo6addr(addr: &mut SockaddrStorage, force: bool, op: u8) -> Result<(), Errno> {
+    let is_any = is_any6addr(addr);
+
+    // SAFETY: Deny connect/sendto to anyaddr with ENETUNREACH.
+    // This is consistent with FreeBSD's net.inet.ip.connect_inaddr_wild sysctl.
+    // 0x3: connect, 0x9: send, 0xb: sendto, 0x10: sendmsg, 0x14: sendmmsg
+    if is_any && matches!(op, 0x3 | 0x9 | 0xb | 0x10 | 0x14) {
+        return Err(Errno::ENETUNREACH);
     }
+
+    if !force && !is_any {
+        return Ok(());
+    }
+
     if let Some(sin6) = addr.as_sockaddr_in6() {
         let port = sin6.port();
         let flowinfo = sin6.flowinfo();
@@ -882,6 +917,8 @@ pub(crate) fn make_lo6addr(addr: &mut SockaddrStorage, force: bool) {
             scope_id,
         ));
     }
+
+    Ok(())
 }
 
 // Return true if `addr` is 0.0.0.0 (Ipv4).
