@@ -2,7 +2,7 @@
 // Syd: rock-solid application kernel
 // src/kernel/open.rs: creat(2), open(2), openat(2), and openat2(2) handlers
 //
-// Copyright (c) 2023, 2024, 2025 Ali Polatel <alip@chesswob.org>
+// Copyright (c) 2023, 2024, 2025, 2026 Ali Polatel <alip@chesswob.org>
 //
 // SPDX-License-Identifier: GPL-3.0
 
@@ -35,6 +35,7 @@ use crate::{
     },
     confine::EIDRM,
     cookie::{safe_memfd_create, safe_openat2, safe_umask, safe_uname},
+    debug,
     err::err2no,
     error,
     fd::{is_dev_kfd, seal_memfd_all},
@@ -84,6 +85,7 @@ pub(crate) fn sys_creat(request: UNotifyEventRequest) -> ScmpNotifResp {
     // Here we omit MUST_PATH in fsflags as path may not exist yet.
     let arg = SysArg {
         path: Some(0),
+        flags: SysFlags::OPENER_CALL,
         fsflags: FsFlags::empty(),
         ..Default::default()
     };
@@ -261,7 +263,7 @@ fn syscall_open_handler(
         }
 
         if sandbox.log_scmp() {
-            crate::debug!("ctx": "open", "op": "init",
+            debug!("ctx": "open", "op": "init",
                 "cap": caps,
                 "sbc": *sandbox.state,
                 "arg": format!("{arg:?}"),
@@ -269,7 +271,7 @@ fn syscall_open_handler(
                 "mod": format!("{mode:?}"),
                 "req": &request);
         } else {
-            crate::debug!("ctx": "open", "op": "init",
+            debug!("ctx": "open", "op": "init",
                 "cap": caps,
                 "sbc": *sandbox.state,
                 "arg": format!("{arg:?}"),
@@ -296,11 +298,11 @@ fn syscall_open_handler(
         let (mut path, magic, _) = request.read_path(&sandbox, arg)?;
 
         if sandbox.log_scmp() {
-            crate::debug!("ctx": "open", "op": "read_path",
+            debug!("ctx": "open", "op": "read_path",
                 "path": &path, "magic": magic,
                 "req": &request);
         } else {
-            crate::debug!("ctx": "open", "op": "read_path",
+            debug!("ctx": "open", "op": "read_path",
                 "path": &path, "magic": magic,
                 "pid": request.scmpreq.pid().as_raw());
         }
@@ -398,7 +400,7 @@ fn syscall_open_handler(
 
                     (true, false, false)
                 }
-                FileType::Mfd => {
+                FileType::Mfd | FileType::MfdSec => {
                     // SAFETY:
                     // 1. We do not encrypt memory fds.
                     // 2. We do not support interruptions on memory fd blocks.
@@ -428,7 +430,7 @@ fn syscall_open_handler(
                     // 1. Do not allow access to block devices.
                     // 2. Restriction may be relaxed with trace/allow_unsafe_mkbdev:1.
                     // 3. Deny with ENOENT for stealth.
-                    if !sandbox.flags.allow_unsafe_mkbdev() {
+                    if !sandbox.options.allow_unsafe_mkbdev() {
                         return Err(Errno::ENOENT);
                     }
 
@@ -472,18 +474,33 @@ fn syscall_open_handler(
         if path.is_char_device() {
             const TTY: &[&[u8]] = &[b"/dev/tty", b"/dev/console"];
             for p in TTY {
-                if path.abs().is_equal(p) {
-                    let real_tty = proc_tty(pid)?;
-                    if !request.is_valid() {
-                        return Err(Errno::ESRCH);
-                    }
-                    path = CanonicalPath::new_tty(real_tty)?;
-                    path_tty = Some(Cow::Borrowed(XPath::from_bytes(p)));
-                    break;
+                if !path.abs().is_equal(p) {
+                    continue;
                 }
+
+                let real_tty = proc_tty(pid)?;
+                if !request.is_valid() {
+                    return Err(Errno::ESRCH);
+                }
+                path = CanonicalPath::new_tty(real_tty)?;
+                path_tty = Some(Cow::Borrowed(XPath::from_bytes(p)));
+                break;
             }
         }
+
         let has_tty = path_tty.is_some();
+        let log_scmp = sandbox.log_scmp();
+        if has_tty {
+            if log_scmp {
+                debug!("ctx": "open", "op": "flip_tty",
+                    "path": &path, "tty": &path_tty,
+                    "req": &request);
+            } else {
+                debug!("ctx": "open", "op": "flip_tty",
+                    "path": &path, "tty": &path_tty,
+                    "pid": request.scmpreq.pid);
+            }
+        }
 
         // SAFETY: Ensure the sandbox process cannot
         // acquire a new controlling terminal other
@@ -498,9 +515,7 @@ fn syscall_open_handler(
         // Sandboxing.
         let orig_caps = caps;
         let caps = sandbox.getcaps(orig_caps); // Get enabled caps.
-
         let mut action = Action::Allow;
-        let mut filter = false;
 
         // Convert /proc/$pid to /proc/self as necessary.
         let mut path_check = path.abs().replace_proc_self(pid);
@@ -515,10 +530,9 @@ fn syscall_open_handler(
                 .enumerate()
             {
                 for cap in caps {
-                    let (new_action, new_filter) = sandbox.check_path(cap, path);
+                    let new_action = sandbox.check_path(cap, path);
                     if new_action >= action {
                         action = new_action;
-                        filter = new_filter;
                         if idx > 0 && new_action > Action::Allow {
                             flip_tty = true;
                         }
@@ -596,13 +610,11 @@ fn syscall_open_handler(
 
         // SAFETY:
         // Check for Filesystem sandboxing as necessary.
-        let log_scmp = sandbox.log_scmp();
         let mut maybe_fs_type = None;
         if action.is_allowing() && sandbox.has_fs() {
-            let fd = path.dir.as_ref().ok_or(Errno::EBADF)?;
-            let fs_type = FsType::get(fd)?;
-            let (action, filter) = sandbox.check_fs(fs_type);
-            if !filter && action >= Action::Warn && log_enabled!(LogLevel::Warn) {
+            let fs_type = FsType::get(path.dir())?;
+            let action = sandbox.check_fs(fs_type);
+            if action.is_logging() {
                 if log_scmp {
                     warn!("ctx": "access", "cap": Capability::CAP_FS, "act": action,
                         "sys": request.syscall, "fs": fs_type, "path": &path_check,
@@ -642,9 +654,11 @@ fn syscall_open_handler(
         let unsafe_open_path = sandbox.flags.allow_unsafe_open_path();
         let filter_machine_id = !sandbox.flags.allow_unsafe_machine_id();
         let filter_proc_status = !sandbox.flags.allow_unsafe_proc_pid_status();
-        let filter_uname = !sandbox.flags.allow_unsafe_uname();
+        let filter_uname = !sandbox.options.allow_unsafe_uname();
         if sandbox.flags.force_cloexec() {
             o_cloexec = true;
+        } else if cfg!(feature = "kcov") && path.abs().is_kcov() {
+            o_cloexec = false;
         }
         let o_rand_fd = sandbox.flags.force_rand_fd();
         let setup_fds = sandbox.crypt_setup();
@@ -652,14 +666,11 @@ fn syscall_open_handler(
         drop(sandbox); // release the read-lock.
 
         // Perform action: allow->emulate, deny->log.
-        if !filter && action >= Action::Warn && log_enabled!(LogLevel::Warn) {
+        if action.is_logging() && log_enabled!(LogLevel::Warn) {
             let fs_type = if let Some(fs_type) = maybe_fs_type {
                 fs_type
             } else {
-                path.dir
-                    .as_ref()
-                    .ok_or(Errno::EBADF)
-                    .and_then(FsType::get)?
+                FsType::get(path.dir())?
             };
             let grp = caps.to_string().to_ascii_lowercase();
             if log_scmp {
@@ -811,6 +822,7 @@ fn syscall_open_handler(
                     && !o_creat
                     && !o_trunc
                     && !o_tmpfl
+                    && path.base().is_empty()
                     && is_dev_kfd(path.dir.as_ref().map(|fd| fd.as_fd()).ok_or(Errno::EBADF)?)
                         .unwrap_or(false)
                 {
@@ -1148,8 +1160,7 @@ fn do_creat(
     safe_umask(mask);
 
     // All set, make the openat2(2) call.
-    let fd = path.dir.as_ref().map(|fd| fd.as_fd()).ok_or(Errno::EBADF)?;
-    safe_openat2(fd, base, how)
+    safe_openat2(path.dir(), base, how)
 }
 
 fn to_oflag(arg: u64, truncate: bool) -> Result<OFlag, Errno> {
@@ -1166,16 +1177,12 @@ fn to_oflag(arg: u64, truncate: bool) -> Result<OFlag, Errno> {
     flags.remove(KEEP_OFLAG);
 
     if truncate {
-        // open, openat truncates invalid flags.
+        // open(2), openat(2) truncates invalid flags.
         return Ok((flags & OFlag::all()) | preserved);
     }
 
-    // openat2 validates invalid flags.
-    let unknown_flags = flags.difference(OFlag::all());
-    if unknown_flags.bits() != 0 {
-        crate::error!("ctx": "open", "op": "invalid_flags",
-            "msg": "passed invalid flags to open flags argument",
-            "flags": format!("{unknown_flags:?}"));
+    // openat2(2) validates invalid flags.
+    if flags.difference(OFlag::all()).bits() != 0 {
         return Err(Errno::EINVAL);
     }
 
