@@ -14,7 +14,6 @@ use nix::{
     fcntl::{AtFlags, OFlag},
     sys::signal::{kill, Signal},
     unistd::Pid,
-    NixPath,
 };
 
 use crate::{
@@ -45,7 +44,7 @@ pub(crate) fn sysenter_exec(
     pid: Pid,
     sandbox: &SandboxGuard,
     info: ptrace_syscall_info,
-) -> Result<(File, ExecutableFile), Errno> {
+) -> Result<(), Errno> {
     let data = if let Some(data) = info.seccomp() {
         data
     } else {
@@ -226,18 +225,13 @@ pub(crate) fn sysenter_exec(
         }
     }
 
-    // Open a new instance of the file for read.
-    // We prefer this over WANT_READ to ensure
-    // we do not share OFD with the sandbox process.
-    assert!(
-        path.base().is_empty(),
-        "BUG: MUST_PATH returned a directory for exec, report a bug!"
-    );
+    // 1. Reopen file as read-only.
+    // 2. Use O_NOCTTY to avoid acquiring controlling terminal.
     let mut file = match path.dir.take() {
         Some(MaybeFd::Owned(fd)) => {
             let pfd = XPathBuf::from_self_fd(fd.as_raw_fd())?;
-            safe_open_msym(PROC_FILE(), &pfd, OFlag::O_RDONLY, ResolveFlag::empty())
-                .map(File::from)?
+            let pfl = OFlag::O_RDONLY | OFlag::O_NOCTTY;
+            safe_open_msym(PROC_FILE(), &pfd, pfl, ResolveFlag::empty()).map(File::from)?
         }
         _ => return Err(Errno::ENOEXEC),
     };
@@ -255,42 +249,17 @@ pub(crate) fn sysenter_exec(
 
     let exe = match ExecutableFile::parse(&file, check_linking) {
         Ok(exe) => exe,
-        Err(ElfError::IoError(err)) => {
-            let errno = err2no(&err);
-            if !sandbox.filter_path(Capability::CAP_EXEC, path.abs()) {
-                error!("ctx": "parse_elf",
-                    "msg": format!("io error: {errno}"),
-                    "pid": pid.as_raw(), "path": path.abs());
-            }
-            return Err(errno);
-        }
-        Err(ElfError::BadMagic) => {
-            if !sandbox.filter_path(Capability::CAP_EXEC, path.abs()) {
-                error!("ctx": "exec", "op": "parse_elf",
-                    "msg": "invalid ELF file",
-                    "pid": pid.as_raw(), "path": path.abs());
-            }
-            return Err(Errno::ENOEXEC);
-        }
-        Err(ElfError::Malformed) => {
-            if !sandbox.filter_path(Capability::CAP_EXEC, path.abs()) {
-                error!("ctx": "exec", "op": "parse_elf",
-                    "msg": "malformed ELF file",
-                    "pid": pid.as_raw(), "path": path.abs());
-            }
-            return Err(Errno::ENOEXEC);
-        }
+        Err(ElfError::IoError(err)) => return Err(err2no(&err)),
+        Err(ElfError::BadMagic | ElfError::Malformed) => return Err(Errno::ENOEXEC),
     };
 
     let is_script = exe == ExecutableFile::Script;
     if is_script && deny_script {
-        if !sandbox.filter_path(Capability::CAP_EXEC, path.abs()) {
-            error!("ctx": "exec", "op": "deny_exec_script",
-                "msg": "script execution denied",
-                "pid": pid.as_raw(), "path": path.abs(),
-                "tip": "configure `trace/deny_exec_script:0'",
-                "exe": format!("{exe}"));
-        }
+        error!("ctx": "exec", "op": "deny_exec_script",
+            "msg": "script execution denied",
+            "pid": pid.as_raw(), "path": path.abs(),
+            "tip": "configure `trace/deny_exec_script:0'",
+            "exe": format!("{exe}"));
         return Err(Errno::EACCES);
     }
 
@@ -304,34 +273,28 @@ pub(crate) fn sysenter_exec(
             }
         )
     {
-        if !sandbox.filter_path(Capability::CAP_EXEC, path.abs()) {
-            error!("ctx": "exec", "op": "check_elf",
-                "msg": "ld.so(8) exec-indirection prevented",
-                "pid": pid.as_raw(), "path": path.abs(),
-                "tip": "configure `trace/allow_unsafe_exec_ldso:1'",
-                "exe": format!("{exe}"));
-        }
+        error!("ctx": "exec", "op": "check_elf",
+            "msg": "ld.so(8) exec-indirection prevented",
+            "pid": pid.as_raw(), "path": path.abs(),
+            "tip": "configure `trace/allow_unsafe_exec_ldso:1'",
+            "exe": format!("{exe}"));
         return Err(Errno::EACCES);
     }
 
     if !is_script && restrict_pie && matches!(exe, ExecutableFile::Elf { pie: false, .. }) {
-        if !sandbox.filter_path(Capability::CAP_EXEC, path.abs()) {
-            error!("ctx": "check_elf", "msg": "ELF is not Position Independent Executable (PIE)",
-                "pid": pid.as_raw(), "path": path.abs(),
-                "tip": "configure `trace/allow_unsafe_exec_nopie:1'",
-                "exe": format!("{exe}"));
-        }
+        error!("ctx": "check_elf", "msg": "ELF is not Position Independent Executable (PIE)",
+            "pid": pid.as_raw(), "path": path.abs(),
+            "tip": "configure `trace/allow_unsafe_exec_nopie:1'",
+            "exe": format!("{exe}"));
         return Err(Errno::EACCES);
     }
 
     if !is_script && restrict_xs && matches!(exe, ExecutableFile::Elf { xs: true, .. }) {
-        if !sandbox.filter_path(Capability::CAP_EXEC, path.abs()) {
-            error!("ctx": "exec", "op": "check_elf",
-                "msg": "ELF has executable stack",
-                "pid": pid.as_raw(), "path": path.abs(),
-                "tip": "configure `trace/allow_unsafe_exec_stack:1'",
-                "exe": format!("{exe}"));
-        }
+        error!("ctx": "exec", "op": "check_elf",
+            "msg": "ELF has executable stack",
+            "pid": pid.as_raw(), "path": path.abs(),
+            "tip": "configure `trace/allow_unsafe_exec_stack:1'",
+            "exe": format!("{exe}"));
         return Err(Errno::EACCES);
     }
 
@@ -345,13 +308,11 @@ pub(crate) fn sysenter_exec(
             }
         )
     {
-        if !sandbox.filter_path(Capability::CAP_EXEC, path.abs()) {
-            error!("ctx": "exec", "op": "check_elf",
-                "msg": "32-bit ELF execution prevented",
-                "pid": pid.as_raw(), "path": path.abs(),
-                "tip": "configure `trace/deny_exec_elf32:0'",
-                "exe": format!("{exe}"));
-        }
+        error!("ctx": "exec", "op": "check_elf",
+            "msg": "32-bit ELF execution prevented",
+            "pid": pid.as_raw(), "path": path.abs(),
+            "tip": "configure `trace/deny_exec_elf32:0'",
+            "exe": format!("{exe}"));
         return Err(Errno::EACCES);
     }
 
@@ -365,13 +326,11 @@ pub(crate) fn sysenter_exec(
             }
         )
     {
-        if !sandbox.filter_path(Capability::CAP_EXEC, path.abs()) {
-            error!("ctx": "exec", "op": "check_elf",
-                "msg": "dynamic linked ELF execution prevented",
-                "pid": pid.as_raw(), "path": path.abs(),
-                "tip": "configure `trace/deny_exec_elf_dynamic:0'",
-                "exe": format!("{exe}"));
-        }
+        error!("ctx": "exec", "op": "check_elf",
+            "msg": "dynamic linked ELF execution prevented",
+            "pid": pid.as_raw(), "path": path.abs(),
+            "tip": "configure `trace/deny_exec_elf_dynamic:0'",
+            "exe": format!("{exe}"));
         return Err(Errno::EACCES);
     }
 
@@ -385,13 +344,11 @@ pub(crate) fn sysenter_exec(
             }
         )
     {
-        if !sandbox.filter_path(Capability::CAP_EXEC, path.abs()) {
-            error!("ctx": "exec", "op": "check_elf",
-                "msg": "static linked ELF execution prevented",
-                "pid": pid.as_raw(), "path": path,
-                "tip": "configure `trace/deny_exec_elf_static:0'",
-                "exe": format!("{exe}"));
-        }
+        error!("ctx": "exec", "op": "check_elf",
+            "msg": "static linked ELF execution prevented",
+            "pid": pid.as_raw(), "path": path,
+            "tip": "configure `trace/deny_exec_elf_static:0'",
+            "exe": format!("{exe}"));
         return Err(Errno::EACCES);
     }
 
@@ -550,5 +507,5 @@ pub(crate) fn sysenter_exec(
             "ip_vma": ip_vma, "sp_vma": sp_vma);
     }
 
-    Ok((file, exe))
+    Ok(())
 }
