@@ -2,7 +2,7 @@
 // Syd: rock-solid application kernel
 // src/kernel/ptrace/mmap.rs: ptrace mmap handlers
 //
-// Copyright (c) 2025 Ali Polatel <alip@chesswob.org>
+// Copyright (c) 2025, 2026 Ali Polatel <alip@chesswob.org>
 //
 // SPDX-License-Identifier: GPL-3.0
 
@@ -22,9 +22,10 @@ use nix::{
 use crate::{
     compat::ResolveFlag,
     config::{PAGE_SIZE, PROC_FILE, PTRACE_DATA_MMAP},
+    cookie::{safe_pidfd_getfd, safe_pidfd_open},
     elf::ExecutableFile,
     error,
-    fd::{fd_status_flags, pidfd_getfd, pidfd_open, PIDFD_THREAD},
+    fd::{fd_status_flags, PIDFD_THREAD},
     kernel::sandbox_path,
     lookup::{safe_open_msym, CanonicalPath},
     path::XPathBuf,
@@ -45,7 +46,7 @@ pub(crate) fn sysenter_mmap(
     pid: Pid,
     sandbox: &SandboxGuard,
     info: ptrace_syscall_info,
-) -> Result<(), Errno> {
+) -> Result<bool, Errno> {
     let data = if let Some(data) = info.seccomp() {
         data
     } else {
@@ -69,7 +70,7 @@ pub(crate) fn sysenter_mmap(
     let mem_max = sandbox.mem_max;
     let mem_vm_max = sandbox.mem_vm_max;
     let mem_act = sandbox.default_action(Capability::CAP_MEM);
-    let restrict_exec_memory = !sandbox.flags.allow_unsafe_exec_memory();
+    let restrict_exec_memory = !sandbox.options.allow_unsafe_exec_memory();
     let restrict_exec_stack = !sandbox.flags.allow_unsafe_exec_stack();
     let restrict_append_only = sandbox.has_append() || sandbox.enabled(Capability::CAP_CRYPT);
 
@@ -81,8 +82,8 @@ pub(crate) fn sysenter_mmap(
         && !restrict_append_only
         && (!mem || (mem_max == 0 && mem_vm_max == 0))
     {
-        // Stop at syscall exit.
-        return Ok(());
+        // Continue system call.
+        return Ok(false);
     }
 
     let check_exec = (exec || force || tpe || restrict_exec_memory || restrict_exec_stack)
@@ -98,8 +99,8 @@ pub(crate) fn sysenter_mmap(
             return Err(Errno::EBADF);
         }
 
-        let pid_fd = pidfd_open(pid, PIDFD_THREAD)?;
-        match pidfd_getfd(pid_fd, remote_fd) {
+        let pid_fd = safe_pidfd_open(pid, PIDFD_THREAD)?;
+        match safe_pidfd_getfd(pid_fd, remote_fd) {
             Ok(fd) => Some(fd),
             Err(_) => return Err(Errno::EBADF),
         }
@@ -224,14 +225,12 @@ pub(crate) fn sysenter_mmap(
                 // https://www.qualys.com/2023/07/19/cve-2023-38408/rce-openssh-forwarded-ssh-agent.txt
                 let exe = ExecutableFile::parse(&mut file, true).or(Err(Errno::EACCES))?;
                 if matches!(exe, ExecutableFile::Elf { xs: true, .. }) {
-                    if !sandbox.filter_path(Capability::CAP_EXEC, path.abs()) {
-                        error!("ctx": "check_lib",
-                            "msg": "library load with executable stack blocked",
-                            "sys": &name, "path": path.abs(),
-                            "tip": "configure `trace/allow_unsafe_exec_stack:1'",
-                            "lib": format!("{exe}"),
-                            "pid": pid.as_raw());
-                    }
+                    error!("ctx": "check_lib",
+                        "msg": "library load with executable stack blocked",
+                        "sys": &name, "path": path.abs(),
+                        "tip": "configure `trace/allow_unsafe_exec_stack:1'",
+                        "lib": format!("{exe}"),
+                        "pid": pid.as_raw());
                     return Err(Errno::EACCES);
                 }
             }
@@ -325,7 +324,8 @@ pub(crate) fn sysenter_mmap(
     if !mem || (mem_max == 0 && mem_vm_max == 0) {
         // (a) Exec and Memory sandboxing are both disabled.
         // (b) Exec granted access, Memory sandboxing is disabled.
-        return Ok(()); // Stop at syscall exit.
+        // Stop at syscall exit if check_exec is true, otherwise continue.
+        return Ok(check_exec);
     }
 
     // Check VmSize
@@ -378,7 +378,8 @@ pub(crate) fn sysenter_mmap(
             return match mem_act {
                 // Allow cannot happen.
                 Action::Allow => unreachable!(),
-                Action::Warn => Ok(()), // Stop at syscall exit.
+                // Stop at syscall exit if check_exec, otherwise continue.
+                Action::Warn => Ok(check_exec),
                 Action::Panic | Action::Deny | Action::Filter => Err(Errno::ENOMEM),
                 //Do NOT panic the main thread!
                 //Action::Panic => panic!(),
@@ -399,8 +400,8 @@ pub(crate) fn sysenter_mmap(
         }
     }
 
-    // Stop at syscall exit.
-    Ok(())
+    // Stop at syscall exit if check_exec is true, otherwise continue.
+    Ok(check_exec)
 }
 
 #[expect(clippy::cognitive_complexity)]
@@ -409,13 +410,7 @@ pub(crate) fn sysexit_mmap(
     info: ptrace_syscall_info,
     sandbox: &SandboxGuard,
 ) -> Result<(), Errno> {
-    if !sandbox.enabled(Capability::CAP_EXEC) {
-        // Exec sandboxing disabled, nothing to do.
-        // Continue process.
-        return Ok(());
-    }
-
-    // Check for successful sigaction exit.
+    // Check for successful mmap exit.
     match ptrace_get_error(pid, info.arch) {
         Ok(None) => {
             // Successful mmap call, validate proc_pid_maps(5).
@@ -453,7 +448,7 @@ pub(crate) fn sysexit_mmap(
 
     for exec in bins {
         let path = &exec.path;
-        let (action, _) = sandbox.check_path(Capability::CAP_EXEC, path);
+        let action = sandbox.check_path(Capability::CAP_EXEC, path);
         if action.is_allowing() {
             continue;
         }
