@@ -33,7 +33,7 @@ use crate::{
     confine::SydMountAttrFlags,
     hash::SydHashSet,
     landlock::{AccessFs, AccessNet},
-    landlock_policy::LandlockPolicy,
+    landlock_policy::{LandlockPolicy, LANDLOCK_ACCESS_FS, LANDLOCK_ACCESS_NET},
     mount::api::MountAttrFlags,
     path::XPathBuf,
     sandbox::{Action, Capability, SANDBOX_CAPS},
@@ -576,15 +576,32 @@ pub fn parse_port_range(input: &str) -> Result<PortRange, Errno> {
 /// Parse an "allow/lock" command: "allow/lock/<access_list><op><arg>"
 /// Returns `LandlockCmd` or `Errno::EINVAL`.
 pub fn parse_landlock_cmd(input: &str) -> Result<LandlockCmd, Errno> {
-    // Inner parser: match prefix, then capture access_part, op, and arg_part.
+    // Match a capability name via LANDLOCK_ACCESS_FS/NET iteration order.
+    fn lock_name(input: &str) -> IResult<&str, &str> {
+        for name in LANDLOCK_ACCESS_FS.keys().rev() {
+            if let Some(rest) = input.strip_prefix(name) {
+                if rest.is_empty() || rest.starts_with([',', '+', '-', '^']) {
+                    return Ok((rest, *name));
+                }
+            }
+        }
+        for name in LANDLOCK_ACCESS_NET.keys().rev() {
+            if let Some(rest) = input.strip_prefix(name) {
+                if rest.is_empty() || rest.starts_with([',', '+', '-', '^']) {
+                    return Ok((rest, *name));
+                }
+            }
+        }
+        Err(nom::Err::Error(Error::new(input, ErrorKind::Tag)))
+    }
+
+    // Inner parser: "allow/lock/" access op arg.
     fn inner(input: &str) -> IResult<&str, (&str, char, &str)> {
         let (rem, (_, access, op, arg)) = (
             tag("allow/lock/"),
-            // access_part: one or more chars until an op character is reached.
-            take_while1(|c: char| !matches!(c, '+' | '-' | '^')),
+            recognize(separated_list1(char(','), lock_name)),
             one_of("+-^"),
-            // arg_part: rest of input, must be non-empty.
-            take_while1(|_| true),
+            take_while1(|_| true), // arg (rest of line, must be non-empty).
         )
             .parse(input)?;
         Ok((rem, (access, op, arg)))
@@ -890,7 +907,8 @@ fn addr_parser(input: &str) -> IResult<&str, &str> {
 //   "any", "any4", "any6",
 //   "local", "local4", "local6",
 //   "loopback", "loopback4", "loopback6",
-//   "linklocal", "linklocal4", "linklocal6"
+//   "linklocal", "linklocal4", "linklocal6",
+//   "multicast", "multicast4", "multicast6"
 // followed by "!" or "@", then <digits>, optionally "-" <digits>.
 fn host_parser(input: &str) -> IResult<&str, &str> {
     // Base aliases (case-insensitive)
@@ -899,6 +917,7 @@ fn host_parser(input: &str) -> IResult<&str, &str> {
         tag_no_case("local"),
         tag_no_case("loopback"),
         tag_no_case("linklocal"),
+        tag_no_case("multicast"),
     ));
     let alias_tuple = (
         alias_base,
@@ -1377,6 +1396,79 @@ mod tests {
         );
     }
 
+    #[test]
+    fn test_parse_landlock_parse_all_x_plus_path() {
+        let cmd = parse_landlock_cmd("allow/lock/all-x+/trusted").unwrap();
+        assert_eq!(cmd.op, LandlockOp::Add);
+        assert_eq!(
+            cmd.filter.first().cloned().unwrap(),
+            LandlockRule::Fs((
+                LandlockPolicy::access_fs_from_set("all-x"),
+                "/trusted".into(),
+            ))
+        );
+    }
+
+    #[test]
+    fn test_parse_landlock_parse_all_x_minus_path() {
+        let cmd = parse_landlock_cmd("allow/lock/all-x-/trusted").unwrap();
+        assert_eq!(cmd.op, LandlockOp::Rem);
+        assert_eq!(
+            cmd.filter.first().cloned().unwrap(),
+            LandlockRule::Fs((
+                LandlockPolicy::access_fs_from_set("all-x"),
+                "/trusted".into(),
+            ))
+        );
+    }
+
+    #[test]
+    fn test_parse_landlock_parse_all_x_caret_path() {
+        let cmd = parse_landlock_cmd("allow/lock/all-x^/trusted").unwrap();
+        assert_eq!(cmd.op, LandlockOp::Rem);
+        assert_eq!(
+            cmd.filter.first().cloned().unwrap(),
+            LandlockRule::Fs((
+                LandlockPolicy::access_fs_from_set("all-x"),
+                "/trusted".into(),
+            ))
+        );
+    }
+
+    #[test]
+    fn test_parse_landlock_parse_all_x_verify_no_exec() {
+        let all_x = LandlockPolicy::access_fs_from_set("all-x");
+        let all = LandlockPolicy::access_fs_from_set("all");
+        assert!(!all_x.contains(AccessFs::Execute));
+        assert!(all.contains(all_x));
+        assert!(!all_x.contains(all));
+        assert_eq!(all & !all_x, AccessFs::Execute);
+    }
+
+    #[test]
+    fn test_parse_landlock_parse_all_x_with_exec_gives_all() {
+        let cmd = parse_landlock_cmd("allow/lock/all-x,exec+/bin").unwrap();
+        assert_eq!(cmd.op, LandlockOp::Add);
+        assert_eq!(
+            cmd.filter.first().cloned().unwrap(),
+            LandlockRule::Fs((LandlockPolicy::access_fs_from_set("all"), "/bin".into(),))
+        );
+    }
+
+    #[test]
+    fn test_parse_landlock_parse_all_x_comma_read() {
+        let cmd = parse_landlock_cmd("allow/lock/all-x,read+/data").unwrap();
+        assert_eq!(cmd.op, LandlockOp::Add);
+        let rule = cmd.filter.first().cloned().unwrap();
+        if let LandlockRule::Fs((access, path)) = rule {
+            assert!(access.contains(AccessFs::ReadFile));
+            assert!(!access.contains(AccessFs::Execute));
+            assert_eq!(path, "/data");
+        } else {
+            panic!("Expected LandlockRule::Fs");
+        }
+    }
+
     #[test]
     fn test_parse_landlock_parse_rpath_plus_path() {
         let cmd = parse_landlock_cmd("allow/lock/rpath+/trusted").unwrap();
