@@ -14,9 +14,10 @@
 
 // We like clean and simple code with documentation.
 // Keep in sync with lib.rs.
+#![forbid(clippy::as_ptr_cast_mut)]
+#![forbid(clippy::cast_ptr_alignment)]
 #![deny(missing_docs)]
 #![deny(clippy::arithmetic_side_effects)]
-#![deny(clippy::as_ptr_cast_mut)]
 #![deny(clippy::as_underscore)]
 #![deny(clippy::assertions_on_result_states)]
 #![deny(clippy::borrow_as_ptr)]
@@ -26,7 +27,6 @@
 #![deny(clippy::cast_possible_truncation)]
 #![deny(clippy::cast_possible_wrap)]
 #![deny(clippy::cast_precision_loss)]
-#![deny(clippy::cast_ptr_alignment)]
 #![deny(clippy::cast_sign_loss)]
 #![deny(clippy::checked_conversions)]
 #![deny(clippy::clear_with_drain)]
@@ -66,6 +66,7 @@ use std::{
 
 use data_encoding::HEXLOWER;
 use libseccomp::{scmp_cmp, ScmpAction, ScmpFilterContext, ScmpSyscall};
+use memchr::arch::all::is_equal;
 use nix::{
     errno::Errno,
     fcntl::OFlag,
@@ -77,10 +78,14 @@ use nix::{
     unistd::{fchdir, getgid, getpid, getuid, isatty, Pid},
 };
 use syd::{
+    bins::{pty::pty_bin_main, tor::tor_bin_main},
     caps,
-    compat::{set_name, waitid, ResolveFlag, WaitStatus},
+    compat::{set_name, set_no_new_privs, waitid, ResolveFlag, WaitStatus},
     config::*,
-    confine::{confine_rlimit_zero, confine_scmp_madvise, confine_scmp_wx_syd, ExportMode},
+    confine::{
+        confine_landlock_scope, confine_rlimit_zero, confine_scmp_madvise, confine_scmp_wx_syd,
+        secure_getenv, ExportMode,
+    },
     err::err2no,
     error,
     fd::{closeexcept, fdclone},
@@ -125,6 +130,19 @@ static GLOBAL: tcmalloc::TCMalloc = tcmalloc::TCMalloc;
 syd::main! {
     use lexopt::prelude::*;
 
+    // Get process name in argv[0] and multicall utilities:
+    //    - syd-pty(1) for PTY sandboxing.
+    //    - syd-tor(1) for Proxy sandboxing.
+    let name = env::args_os().next();
+    if let Some(name) = name {
+        let name = name.as_bytes();
+        if is_equal(name, b"syd-pty") {
+            return Ok(pty_bin_main());
+        } else if is_equal(name, b"syd-tor") {
+            return Ok(tor_bin_main());
+        }
+    }
+
     // Initialize logging.
     log_init(LogLevel::Warn, Some(libc::STDERR_FILENO))?;
 
@@ -189,6 +207,24 @@ syd::main! {
         }
     }
 
+    // Set NO_NEW_PRIVS as early as possible.
+    set_no_new_privs()?;
+
+    // Apply a landlock(7) scope sandbox to restrict:
+    // 1. ptrace(2) attach outside landlock(7).
+    // 2. Signal send outside landlock(7).
+    // 3. We leave path and network restrictions for Landlock
+    //    to be configured by the user using Lock sandboxing.
+    // 4. We do this before memfd-reexec to add an additional
+    //    guard against proc(5) havoc.
+    if let Err(errno) = confine_landlock_scope() {
+        error!("ctx": "landlock_scope",
+            "err": errno as i32,
+            "msg": format!("landlock scope failed: {errno}"),
+            "tip": "submit a bug report");
+        return Err(errno.into());
+    }
+
     // Guard against CVE-2019-5736:
     // Copy /proc/self/exe in an anonymous fd (created via memfd_create), seal it and re-execute it.
     // See:
@@ -241,7 +277,7 @@ syd::main! {
             Some(val) => env::set_var("SYD_RUST_BACKTRACE", val),
             None => env::remove_var("SYD_RUST_BACKTRACE"),
         };
-        if env::var_os(ENV_SKIP_SCMP).is_none() {
+        if secure_getenv(ENV_SKIP_SCMP).is_none() {
             env::set_var("RUST_BACKTRACE", "0");
         }
         env::set_var(ENV_RAND, get_at_random_hex(false));
@@ -350,7 +386,7 @@ syd::main! {
             .map(|raw| raw.peek().is_none())
             .unwrap_or(true)
     {
-        sandbox.parse_profile("user")?;
+        sandbox.parse_profile(b"user")?;
         true
     } else {
         false
@@ -435,9 +471,9 @@ syd::main! {
             }
             Short('m') => {
                 let cmd = parser.value().map(XPathBuf::from)?;
-                if sandbox.locked() {
+                if sandbox.is_locked() {
                     eprintln!("Failed to execute magic command `{cmd}': sandbox locked!");
-                    return Err(Errno::EBUSY.into());
+                    return Err(Errno::EPERM.into());
                 } else {
                     sandbox.config(&cmd.to_string())?;
                 }
@@ -447,14 +483,14 @@ syd::main! {
                     .ok()
                     .and_then(|ostr| ostr.into_string().ok())
                     .ok_or(Errno::EINVAL)?;
-                if sandbox.locked() {
+                if sandbox.is_locked() {
                     eprintln!("Failed to set sandbox timeout: sandbox locked!");
-                    return Err(Errno::EBUSY.into());
+                    return Err(Errno::EPERM.into());
                 } else {
                     sandbox.config(&format!("timeout:{tmout}"))?;
                 }
             }
-            Short('x') => sandbox.parse_profile("trace")?,
+            Short('x') => sandbox.parse_profile(b"trace")?,
             Short('f') => {
                 // Login shell compatibility:
                 // Parse user profile as necessary.
@@ -479,9 +515,9 @@ syd::main! {
             }
             Short('P') => {
                 let path = parser.value().map(XPathBuf::from)?;
-                if sandbox.locked() {
+                if sandbox.is_locked() {
                     eprintln!("Failed to parse config file `{path}': sandbox locked!");
-                    return Err(Errno::EBUSY.into());
+                    return Err(Errno::EPERM.into());
                 }
                 sandbox.parse_config_file(&path)?;
             }
@@ -489,11 +525,11 @@ syd::main! {
              * It's undocumented. */
             Short('p') | Long("profile") => {
                 let profile = parser.value()?.parse::<String>()?;
-                if sandbox.locked() {
+                if sandbox.is_locked() {
                     eprintln!("Failed to parse profile `{profile}': sandbox locked!");
-                    return Err(Errno::EBUSY.into());
+                    return Err(Errno::EPERM.into());
                 }
-                sandbox.parse_profile(&profile)?;
+                sandbox.parse_profile(profile.as_bytes())?;
             }
 
             /*
@@ -557,8 +593,8 @@ syd::main! {
         env::remove_var(ENV_DUMP_SCMP);
     }
 
-    if user_parse && !user_done && !sandbox.locked() {
-        sandbox.parse_profile("user")?;
+    if user_parse && !user_done && !sandbox.is_locked() {
+        sandbox.parse_profile(b"user")?;
     }
 
     // Prepare the command to execute, which may be a login shell.
@@ -580,7 +616,7 @@ syd::main! {
     // Skip ignoring signals with default action Core,
     // if trace/allow_unsafe_prlimit:1 is set at startup.
     let mut opts = IgnoreSignalOpts::SkipIgnoreAlarm;
-    if sandbox.flags.allow_unsafe_prlimit() {
+    if sandbox.options.allow_unsafe_prlimit() {
         opts.insert(IgnoreSignalOpts::SkipIgnoreCoreDump);
     }
     ignore_signals(opts).inspect_err(|errno| {
@@ -594,16 +630,16 @@ syd::main! {
 
     // SAFETY: We cannot support NEWPID without NEWNS.
     // ie, pid namespace must have its own private /proc.
-    if sandbox.flags.unshare_pid() {
+    if sandbox.options.unshare_pid() {
         sandbox.set_unshare_mount(true);
     }
-    if sandbox.flags.unshare_mount() {
+    if sandbox.options.unshare_mount() {
         sandbox.set_unshare_pid(true);
     }
 
-    let has_ns_user = sandbox.flags.unshare_user();
-    let has_pid_max = sandbox.flags.unshare_pid() && sandbox.has_pid() && sandbox.pid_max > 0;
-    let has_ns_time = sandbox.flags.unshare_time();
+    let has_ns_user = sandbox.options.unshare_user();
+    let has_pid_max = sandbox.options.unshare_pid() && sandbox.has_pid() && sandbox.pid_max > 0;
+    let has_ns_time = sandbox.options.unshare_time();
 
     // Save original UID/GID to map inside new user namespace.
     let (uid, gid) = if has_ns_user {
@@ -654,7 +690,7 @@ syd::main! {
         && isatty(stdin()).unwrap_or(false)
         && isatty(stdout()).unwrap_or(false)
     {
-        let pty_debug = env::var_os("SYD_PTY_DEBUG").is_some();
+        let pty_debug = secure_getenv("SYD_PTY_DEBUG").is_some();
         let pty_child = pty_setup(sandbox.pty_ws_x(), sandbox.pty_ws_y(), pty_debug)?;
 
         let mut buf = itoa::Buffer::new();
@@ -666,7 +702,7 @@ syd::main! {
         None
     };
 
-    let proxy_debug = env::var_os("SYD_TOR_DEBUG").is_some();
+    let proxy_debug = secure_getenv("SYD_TOR_DEBUG").is_some();
     let proxy = if sandbox.has_proxy() {
         // sandbox/proxy:on implies unshare/net:1.
         sandbox.set_unshare_net(true);
@@ -687,7 +723,7 @@ syd::main! {
     // include the Syd process into the pid namespace as well
     // such that the sandbox process and syd have the identical
     // view of /proc.
-    let namespaces = sandbox.flags.namespaces();
+    let namespaces = sandbox.options.namespaces();
     if namespaces == 0 {
         // Drop /proc fd which may be open due to is_rbash.
         drop(fd_proc);
@@ -703,7 +739,9 @@ syd::main! {
             Ok(code) => Ok(ExitCode::from(code)),
             Err(error) => {
                 let errno = error.errno().unwrap_or(Errno::ENOSYS);
-                eprintln!("{error:?}");
+                error!("ctx": "run", "op": "run_supervisor",
+                    "msg": format!("failed to run supervisor: {error:?}"),
+                    "tip": "check with SYD_LOG=debug and/or submit a bug report");
                 Ok(ExitCode::from(u8::try_from(errno as i32).unwrap_or(127)))
             }
         };
@@ -734,7 +772,7 @@ syd::main! {
     #[expect(clippy::disallowed_methods)]
     if has_ns_user {
         let fd_proc = fd_proc.as_ref().unwrap();
-        ns_setup_user(fd_proc, uid.unwrap(), gid.unwrap(), sandbox.flags.map_root())?;
+        ns_setup_user(fd_proc, uid.unwrap(), gid.unwrap(), sandbox.options.map_root())?;
     }
 
     // Set up PID namespace.
@@ -753,19 +791,19 @@ syd::main! {
     drop(fd_proc); // drop /proc fd.
 
     // Set up network namespace.
-    if sandbox.flags.unshare_net() {
+    if sandbox.options.unshare_net() {
         ns_setup_net(proxy.as_ref(), sandbox.proxy_addr, sandbox.proxy_port, proxy_debug)?;
     }
     drop(proxy); // close syd-tor end.
 
     // Set up UTS namespace.
-    if sandbox.flags.unshare_uts() {
+    if sandbox.options.unshare_uts() {
         ns_setup_uts(sandbox.hostname.as_deref(), sandbox.domainname.as_deref())?;
     }
 
     // Disable Speculative Store Bypass mitigations
     // for trace/allow_unsafe_exec_speculative:1
-    let ssb = sandbox.flags.allow_unsafe_exec_speculative();
+    let ssb = sandbox.options.allow_unsafe_exec_speculative();
 
     // Prepare stack for new Syd process.
     // SAFETY: Heap-allocate the clone stack to avoid caller's
