@@ -2,7 +2,7 @@
 // Syd: rock-solid application kernel
 // src/wordexp.rs: Interface for libc's wordexp(3).
 //
-// Copyright (c) 2024, 2025 Ali Polatel <alip@chesswob.org>
+// Copyright (c) 2024, 2025, 2026 Ali Polatel <alip@chesswob.org>
 //
 // SPDX-License-Identifier: GPL-3.0
 
@@ -44,7 +44,7 @@ use nix::{
 use crate::{
     compat::{pipe2_raw, set_pdeathsig, waitid, MFdFlags, WaitStatus},
     config::{MINI_STACK_SIZE, *},
-    confine::{confine_mdwe, confine_rlimit, CLONE_NEWTIME},
+    confine::{confine_mdwe, confine_rlimit, secure_getenv, CLONE_NEWTIME},
     cookie::safe_memfd_create,
     debug,
     err::err2no,
@@ -61,7 +61,7 @@ use crate::{
         util::{mount_bind, mount_fs, set_root_mount_propagation},
     },
     path::PATH_MAX,
-    unshare::{GidMap, UidMap},
+    proc::{proc_map_user, proc_open},
     xpath, XPathBuf,
 };
 
@@ -561,7 +561,7 @@ impl WordExp<'_> {
     #[expect(clippy::cognitive_complexity)]
     #[expect(clippy::disallowed_methods)]
     pub fn confine() {
-        if env::var_os(ENV_SKIP_SCMP).is_some() {
+        if secure_getenv(ENV_SKIP_SCMP).is_some() {
             return;
         }
 
@@ -582,101 +582,7 @@ impl WordExp<'_> {
 
         // SAFETY: set up namespace isolation.
         // continue on errors as unprivileged userns may not be supported.
-        if unshare(
-            CloneFlags::CLONE_NEWUSER
-                | CloneFlags::CLONE_NEWCGROUP
-                | CloneFlags::CLONE_NEWIPC
-                | CloneFlags::CLONE_NEWNET
-                | CloneFlags::CLONE_NEWNS
-                | CloneFlags::CLONE_NEWPID
-                | CloneFlags::CLONE_NEWUTS
-                | CLONE_NEWTIME,
-        )
-        .is_ok()
-        {
-            debug!("ctx": "expand",
-                "msg": "created and entered into new user, mount, pid, network, cgroup, ipc, uts, and time namespaces");
-            // SAFETY: map current user in new user namespace.
-            let uid_buf = {
-                let uid_maps = vec![
-                    UidMap {
-                        inside_uid: uid.into(),
-                        outside_uid: uid.into(),
-                        count: 1,
-                    }, // Map the current user.
-                ];
-                let mut buf = Vec::new();
-                for map in uid_maps {
-                    writeln!(
-                        &mut buf,
-                        "{} {} {}",
-                        map.inside_uid, map.outside_uid, map.count
-                    )
-                    .expect("write uid_map");
-                }
-                buf
-            };
-            let gid_buf = {
-                let gid_maps = vec![
-                    GidMap {
-                        inside_gid: gid.into(),
-                        outside_gid: gid.into(),
-                        count: 1,
-                    }, // Map the current group.
-                ];
-                let mut buf = Vec::new();
-                for map in gid_maps {
-                    writeln!(
-                        &mut buf,
-                        "{} {} {}",
-                        map.inside_gid, map.outside_gid, map.count
-                    )
-                    .expect("write gid_map");
-                }
-                buf
-            };
-            // Write "deny" to /proc/self/setgroups before writing to gid_map.
-            File::create("/proc/self/setgroups")
-                .and_then(|mut f| f.write_all(b"deny"))
-                .expect("deny setgroups");
-            File::create("/proc/self/gid_map")
-                .and_then(|mut f| f.write_all(&gid_buf[..]))
-                .expect("map current group");
-            debug!("ctx": "expand",
-                "msg": format!("mapped current group {gid} into new user namespace"));
-            File::create("/proc/self/uid_map")
-                .and_then(|mut f| f.write_all(&uid_buf[..]))
-                .expect("map current user");
-            debug!("ctx": "expand",
-                "msg": format!("mapped current user {uid} into new user namespace"));
-
-            // SAFETY: remount rootfs as readonly,nosuid,nodev,nosymfollow
-            // Careful here, unshare(2) may be available but mount(2) may not be,
-            // so we must handle mount errors gracefully.
-            let mut flags = MountAttrFlags::MOUNT_ATTR_RDONLY
-                | MountAttrFlags::MOUNT_ATTR_NOSUID
-                | MountAttrFlags::MOUNT_ATTR_NODEV
-                | MountAttrFlags::MOUNT_ATTR_NOSYMFOLLOW;
-
-            if set_root_mount_propagation(MsFlags::MS_PRIVATE).is_ok() {
-                debug!("ctx": "expand",
-                    "msg": "set mount propagation to private in new mount namespace");
-                open(
-                    "/",
-                    OFlag::O_CLOEXEC | OFlag::O_PATH | OFlag::O_DIRECTORY,
-                    Mode::empty(),
-                )
-                .and_then(|root| mount_bind(&root, &root, flags))
-                .expect("remount rootfs");
-                debug!("ctx": "expand",
-                    "msg": "remounted root with readonly, nosuid, nodev, and nosymfollow options in new mount namespace");
-                // SAFETY: mount private procfs
-                // pid=1 is required to exist before this.
-                flags.remove(MountAttrFlags::MOUNT_ATTR_NOSYMFOLLOW);
-                flags.insert(MountAttrFlags::MOUNT_ATTR_NOEXEC);
-                Self::mount_proc(flags);
-            }
-        }
+        let _ = Self::setup_namespaces(uid, gid);
 
         // SAFETY: Landlock: confine filesystem as read-only.
         // continue on errors as Landlock may not be supported.
@@ -863,7 +769,8 @@ impl WordExp<'_> {
 
         for sysname in WORDEXP_SYSCALLS
             .iter()
-            .chain(GET_ID_SYSCALLS)
+            .chain(FUTEX_SYSCALLS)
+            .chain(GETID_SYSCALLS)
             .chain(VDSO_SYSCALLS)
         {
             if let Ok(syscall) = ScmpSyscall::from_name(sysname) {
@@ -878,6 +785,55 @@ impl WordExp<'_> {
             "msg": "loaded seccomp filter");
     }
 
+    #[expect(clippy::cognitive_complexity)]
+    #[expect(clippy::disallowed_methods)]
+    fn setup_namespaces(uid: Uid, gid: Gid) -> Result<(), Errno> {
+        unshare(
+            CloneFlags::CLONE_NEWUSER
+                | CloneFlags::CLONE_NEWCGROUP
+                | CloneFlags::CLONE_NEWIPC
+                | CloneFlags::CLONE_NEWNET
+                | CloneFlags::CLONE_NEWNS
+                | CloneFlags::CLONE_NEWPID
+                | CloneFlags::CLONE_NEWUTS
+                | CLONE_NEWTIME,
+        )?;
+        debug!("ctx": "expand",
+            "msg": "created and entered into new user, mount, pid, network, cgroup, ipc, uts, and time namespaces");
+
+        // Set up UID/GID mapping in new user namespace.
+        proc_map_user(proc_open()?, uid, gid, false /*map_root*/)?;
+
+        // SAFETY: Remount rootfs as readonly,nosuid,nodev,nosymfollow.
+        let mut flags = MountAttrFlags::MOUNT_ATTR_RDONLY
+            | MountAttrFlags::MOUNT_ATTR_NOSUID
+            | MountAttrFlags::MOUNT_ATTR_NODEV
+            | MountAttrFlags::MOUNT_ATTR_NOSYMFOLLOW;
+
+        // Set mount propagation to private.
+        set_root_mount_propagation(MsFlags::MS_PRIVATE)?;
+        debug!("ctx": "expand",
+            "msg": "set mount propagation to private in new mount namespace");
+
+        // Remount root.
+        open(
+            "/",
+            OFlag::O_CLOEXEC | OFlag::O_PATH | OFlag::O_DIRECTORY | OFlag::O_NOFOLLOW,
+            Mode::empty(),
+        )
+        .and_then(|root| mount_bind(&root, &root, flags))?;
+        debug!("ctx": "expand",
+            "msg": "remounted root with readonly, nosuid, nodev, and nosymfollow options in new mount namespace");
+
+        // SAFETY: Mount private procfs.
+        // pid=1 is required to exist before this.
+        flags.remove(MountAttrFlags::MOUNT_ATTR_NOSYMFOLLOW);
+        flags.insert(MountAttrFlags::MOUNT_ATTR_NOEXEC);
+        Self::mount_proc(flags);
+
+        Ok(())
+    }
+
     #[expect(clippy::cognitive_complexity)]
     #[expect(clippy::disallowed_methods)]
     fn mount_proc(flags: MountAttrFlags) {
