@@ -2,12 +2,11 @@
 // Syd: rock-solid application kernel
 // src/workers/int.rs: `syd_int' interrupter thread
 //
-// Copyright (c) 2024, 2025 Ali Polatel <alip@chesswob.org>
+// Copyright (c) 2024, 2025, 2026 Ali Polatel <alip@chesswob.org>
 //
 // SPDX-License-Identifier: GPL-3.0
 
 use std::{
-    env,
     os::fd::{FromRawFd, OwnedFd, RawFd},
     sync::{
         atomic::{AtomicBool, Ordering},
@@ -28,7 +27,10 @@ use crate::{
     alert,
     cache::SysInterrupt,
     config::*,
-    confine::{confine_scmp_madvise, confine_scmp_write, scmp_add_setid_rules, ExportMode},
+    confine::{
+        confine_scmp_fcntl, confine_scmp_madvise, confine_scmp_prctl, confine_scmp_write,
+        confine_scmp_wx_syd, scmp_add_setid_rules, secure_getenv, ExportMode,
+    },
     cookie::{CookieIdx, SYSCOOKIE_POOL},
     err::{err2no, scmp2no, SydJoinHandle, SydResult},
     error,
@@ -37,7 +39,7 @@ use crate::{
     info,
     proc::{proc_status_open, proc_status_read},
     retry::retry_on_eintr,
-    sandbox::Flags,
+    sandbox::Options,
     sigset::SydSigSet,
     workers::WorkerCache,
 };
@@ -76,7 +78,7 @@ impl Serialize for InterruptSource {
 #[derive(Clone)]
 pub(crate) struct Interrupter {
     seccomp_fd: RawFd,
-    flags: Flags,
+    options: Options,
 
     transit_uids: Vec<(Uid, Uid)>,
     transit_gids: Vec<(Gid, Gid)>,
@@ -88,14 +90,14 @@ pub(crate) struct Interrupter {
 impl Interrupter {
     pub(crate) fn new(
         seccomp_fd: RawFd,
-        flags: Flags,
+        options: Options,
         transit_uids: &[(Uid, Uid)],
         transit_gids: &[(Gid, Gid)],
         should_exit: Arc<AtomicBool>,
         cache: Arc<WorkerCache>,
     ) -> Self {
         Self {
-            flags,
+            options,
             seccomp_fd,
             should_exit,
             cache,
@@ -112,9 +114,9 @@ impl Interrupter {
             .spawn(move || {
                 // SAFETY: We use exit_group(2) here to bail,
                 // because this unsharing is a critical safety feature.
-                if let Err(errno) = unshare(CloneFlags::CLONE_FS | CloneFlags::CLONE_FILES) {
+                if let Err(errno) = unshare(CloneFlags::CLONE_FS | CloneFlags::CLONE_FILES | CloneFlags::CLONE_SYSVSEM) {
                     alert!("ctx": "boot", "op": "unshare_interrupt_thread",
-                        "msg": format!("failed to unshare(CLONE_FS|CLONE_FILES): {errno}"),
+                        "msg": format!("failed to unshare(CLONE_FS|CLONE_FILES|CLONE_SYSVSEM): {errno}"),
                         "err": errno as i32);
                     std::process::exit(101);
                 }
@@ -163,7 +165,7 @@ impl Interrupter {
 
                 // Honour dry-run when exporting.
                 let dry_run =
-                    env::var_os(ENV_SKIP_SCMP).is_some() || ExportMode::from_env().is_some();
+                    secure_getenv(ENV_SKIP_SCMP).is_some() || ExportMode::from_env().is_some();
 
                 // Confine `syd_int' thread.
                 if !dry_run {
@@ -172,7 +174,7 @@ impl Interrupter {
                     let ctx = match Self::prepare_confine(
                         self.seccomp_fd,
                         tgid,
-                        self.flags,
+                        self.options,
                         &self.transit_uids,
                         &self.transit_gids,
                         false,
@@ -199,8 +201,8 @@ impl Interrupter {
                     }
 
                     let safe_setid = self
-                        .flags
-                        .intersects(Flags::FL_ALLOW_SAFE_SETUID | Flags::FL_ALLOW_SAFE_SETGID);
+                        .options
+                        .intersects(Options::OPT_ALLOW_SAFE_SETUID | Options::OPT_ALLOW_SAFE_SETGID);
                     info!("ctx": "confine", "op": "confine_int_thread",
                         "msg": format!("interrupt thread confined with{} SROP mitigation",
                             if safe_setid { "out" } else { "" }));
@@ -394,7 +396,7 @@ impl Interrupter {
     pub(crate) fn prepare_confine(
         seccomp_fd: RawFd,
         tgid: Pid,
-        flags: Flags,
+        options: Options,
         transit_uids: &[(Uid, Uid)],
         transit_gids: &[(Gid, Gid)],
         _dry_run: bool,
@@ -413,7 +415,7 @@ impl Interrupter {
 
         // Disable Speculative Store Bypass mitigations
         // with trace/allow_unsafe_exec_speculative:1
-        ctx.set_ctl_ssb(flags.allow_unsafe_exec_speculative())?;
+        ctx.set_ctl_ssb(options.allow_unsafe_exec_speculative())?;
 
         // DO NOT synchronize filter to all threads.
         // Other threads will self-confine.
@@ -521,49 +523,13 @@ impl Interrupter {
         }
 
         // Allow safe fcntl(2) utility calls.
-        for sysname in ["fcntl", "fcntl64"] {
-            let syscall = match ScmpSyscall::from_name(sysname) {
-                Ok(syscall) => syscall,
-                Err(_) => {
-                    info!("ctx": "confine", "op": "allow_int_syscall",
-                        "msg": format!("invalid or unsupported syscall {sysname}"));
-                    continue;
-                }
-            };
-
-            for op in INT_FCNTL_OPS {
-                ctx.add_rule_conditional(ScmpAction::Allow, syscall, &[scmp_cmp!($arg1 == *op)])?;
-            }
-        }
+        confine_scmp_fcntl(&mut ctx, INT_FCNTL_OPS)?;
 
         // Allow safe prctl(2) operations.
-        let sysname = "prctl";
-        if let Ok(syscall) = ScmpSyscall::from_name(sysname) {
-            for (_, op) in INT_PRCTL_OPS {
-                ctx.add_rule_conditional(ScmpAction::Allow, syscall, &[scmp_cmp!($arg0 == *op)])?;
-            }
-        } else {
-            info!("ctx": "confine", "op": "allow_int_syscall",
-                "msg": format!("invalid or unsupported syscall {sysname}"));
-        }
+        confine_scmp_prctl(&mut ctx, INT_PRCTL_OPS)?;
 
         // Prevent executable memory.
-        const PROT_EXEC: u64 = libc::PROT_EXEC as u64;
-        for sysname in ["mmap", "mmap2", "mprotect"] {
-            match ScmpSyscall::from_name(sysname) {
-                Ok(syscall) => {
-                    ctx.add_rule_conditional(
-                        ScmpAction::Allow,
-                        syscall,
-                        &[scmp_cmp!($arg2 & PROT_EXEC == 0)],
-                    )?;
-                }
-                Err(_) => {
-                    info!("ctx": "confine", "op": "allow_int_syscall",
-                        "msg": format!("invalid or unsupported syscall {sysname}"));
-                }
-            }
-        }
+        confine_scmp_wx_syd(&mut ctx)?;
 
         // Allow writes to the log-fd.
         // No proc_pid_mem(5) access required here.
@@ -572,34 +538,16 @@ impl Interrupter {
         // Allow safe madvise(2) advice.
         confine_scmp_madvise(&mut ctx)?;
 
-        // Allow safe system calls.
-        for sysname in INT_SYSCALLS.iter().chain(VDSO_SYSCALLS) {
-            match ScmpSyscall::from_name(sysname) {
-                Ok(syscall) => {
-                    ctx.add_rule(ScmpAction::Allow, syscall)?;
-                }
-                Err(_) => {
-                    info!("ctx": "confine", "op": "allow_int_syscall",
-                        "msg": format!("invalid or unsupported syscall {sysname}"));
-                }
-            }
-        }
-
-        // Allow futex system calls.
-        for sysname in FUTEX_SYSCALLS {
-            match ScmpSyscall::from_name(sysname) {
-                Ok(syscall) => {
-                    ctx.add_rule(ScmpAction::Allow, syscall)?;
-                }
-                Err(_) => {
-                    info!("ctx": "confine", "op": "allow_int_syscall",
-                        "msg": format!("invalid or unsupported syscall {sysname}"));
-                }
-            }
-        }
-
-        // Allow getid system calls.
-        for sysname in GET_ID_SYSCALLS {
+        // Allow safe, futex and getid system calls.
+        //
+        // KCOV_SYSCALLS is empty in case `kcov` feature is disabled.
+        for sysname in INT_SYSCALLS
+            .iter()
+            .chain(FUTEX_SYSCALLS)
+            .chain(GETID_SYSCALLS)
+            .chain(KCOV_SYSCALLS)
+            .chain(VDSO_SYSCALLS)
+        {
             match ScmpSyscall::from_name(sysname) {
                 Ok(syscall) => {
                     ctx.add_rule(ScmpAction::Allow, syscall)?;
@@ -612,8 +560,8 @@ impl Interrupter {
         }
 
         // Allow UID/GID changing system calls as necessary.
-        let safe_setuid = flags.allow_safe_setuid();
-        let safe_setgid = flags.allow_safe_setgid();
+        let safe_setuid = options.allow_safe_setuid();
+        let safe_setgid = options.allow_safe_setgid();
         if safe_setuid || safe_setgid {
             scmp_add_setid_rules(
                 "int",
