@@ -2,7 +2,7 @@
 // Syd: rock-solid application kernel
 // src/workers/mod.rs: Worker threads implementation
 //
-// Copyright (c) 2024, 2025 Ali Polatel <alip@chesswob.org>
+// Copyright (c) 2024, 2025, 2026 Ali Polatel <alip@chesswob.org>
 // Based in part upon rusty_pool which is:
 //     Copyright (c) Robin Friedli <robinfriedli@icloud.com>
 //     SPDX-License-Identifier: Apache-2.0
@@ -11,11 +11,11 @@
 
 use std::{
     collections::hash_map::Entry,
-    fs::File,
     option::Option,
     sync::atomic::{AtomicUsize, Ordering},
 };
 
+use libc::c_long;
 use nix::{
     errno::Errno,
     sys::signal::{SigSet, Signal},
@@ -25,10 +25,9 @@ use nix::{
 use crate::{
     cache::{
         ptrace_map_new, signal_map_new, sys_interrupt_map_new, sys_result_map_new, unix_map_new,
-        ExecResult, PtraceMap, SignalMap, SysInterrupt, SysInterruptMap, SysResultMap, UnixMap,
+        PtraceMap, SignalMap, SysInterrupt, SysInterruptMap, SysResultMap, UnixMap,
     },
     confine::ScmpNotifReq,
-    elf::ExecutableFile,
     fs::{block_signal, sigtimedpoll, unblock_signal},
     retry::retry_on_eintr,
     sigset::SydSigSet,
@@ -154,17 +153,17 @@ impl WorkerCache {
         map.retain(|_, &mut pid| pid != tgid)
     }
 
-    // Record a chdir result.
-    pub(crate) fn add_chdir(&self, pid: Pid) {
+    // Record a chdir(2) pid and syscall number (chdir or fchdir).
+    pub(crate) fn add_chdir(&self, pid: Pid, scno: c_long) {
         self.sysres_map
             .trace_chdir
             .lock()
             .unwrap_or_else(|err| err.into_inner())
-            .insert(pid);
+            .insert(pid, scno);
     }
 
     // Query, remove and return a chdir result.
-    pub(crate) fn get_chdir(&self, pid: Pid) -> bool {
+    pub(crate) fn get_chdir(&self, pid: Pid) -> Option<c_long> {
         self.sysres_map
             .trace_chdir
             .lock()
@@ -172,17 +171,17 @@ impl WorkerCache {
             .remove(&pid)
     }
 
-    // Record a mmap(2) pid.
-    pub(crate) fn add_mmap(&self, pid: Pid) {
+    // Record a mmap(2) pid and syscall number (mmap or mmap2).
+    pub(crate) fn add_mmap(&self, pid: Pid, scno: c_long) {
         self.sysres_map
             .trace_mmap
             .lock()
             .unwrap_or_else(|err| err.into_inner())
-            .insert(pid);
+            .insert(pid, scno);
     }
 
     // Query, remove and return true if found.
-    pub(crate) fn get_mmap(&self, pid: Pid) -> bool {
+    pub(crate) fn get_mmap(&self, pid: Pid) -> Option<c_long> {
         self.sysres_map
             .trace_mmap
             .lock()
@@ -208,26 +207,6 @@ impl WorkerCache {
             .remove_entry(&pid)
     }
 
-    // Record a execv result.
-    pub(crate) fn add_exec(&self, pid: Pid, exe: ExecutableFile, file: File) {
-        let result = ExecResult { exe, file };
-
-        self.sysres_map
-            .trace_execv
-            .lock()
-            .unwrap_or_else(|err| err.into_inner())
-            .insert(pid, result);
-    }
-
-    // Query, remove and return a exec result.
-    pub(crate) fn get_exec(&self, pid: Pid) -> Option<(Pid, ExecResult)> {
-        self.sysres_map
-            .trace_execv
-            .lock()
-            .unwrap_or_else(|err| err.into_inner())
-            .remove_entry(&pid)
-    }
-
     // Add a restarting signal.
     pub(crate) fn add_sig_restart(&self, request_tgid: Pid, sig: libc::c_int) -> Result<(), Errno> {
         let mut map = self
@@ -341,9 +320,6 @@ impl WorkerCache {
 
         // Remove preexisting chdir record for pid.
         let _ = self.get_chdir(pid);
-
-        // Remove preexisting exec record for pid.
-        let _ = self.get_exec(pid);
     }
 }
 
